# -*- coding: utf-8 -*-
"""
created 2020/05/28, Markus K\"uhbach, m.kuehbach@mpie.de
Python class to be used in Jupyter notebooks for strongly automatized reporting
of results from the paraprobe-ranger tool
"""

import os, sys, glob, subprocess
from pathlib import Path
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import periodictable as pse
import h5py

from PARAPROBE_Autoreporter_Numerics import *
from PARAPROBE_Autoreporter_CorpDesign import *
from PARAPROBE_Autoreporter_DecodeIontypeHash import *
from PARAPROBE_Autoreporter_Plots import *

from PARAPROBE_Autoreporter_MetadataTranscoder import *
from PARAPROBE_Autoreporter_MetadataRanger import *

#dictionary of settings to read from a paraprobe-spatstat runs which are worth to be reported
#paraprobe settings always have the same format within an paraprobe H5 file, they are
#3x1 matrices of strings the [0].decode() gives the value, [1].decode() the unit, [2].decode() a description
config_ranger = {} #there are none

class autoreporter_ranger():
    
    def __init__(self, fn, *args, **kwargs):
        #self.simid = simid
        self.toolname = 'Ranger'
        self.h5src = fn
        
        #add here all functions which parse by default results from the source
        #self.get_iontypes()
        #self.get_composition()
        self.config = config_ranger
        #dictionary keeping references to the latest results
        self.results = {}        
        #add here all functions which parse by default results from the source
        self.get_tool_config()
        
    def get_tool_config(self):
        fn = self.h5src
        hf = h5py.File( fn, 'r' )
        self.config = config_ranger
        keys = list(hf['/SoftwareDetails/Metadata/Keywords/'])
        self.config = {}
        for key in keys:
            print(key)
            tmp = hf['/SoftwareDetails/Metadata/Keywords/' + key][:]
            self.config[key] = [ tmp[0].decode(), tmp[1].decode(), tmp[2].decode() ]
        self.results['Config'] = self.config 
            
    def get_iontypes(self):
        """read which ion types ranger detected on h5src
        """
        #fn = 'PARAPROBE.Transcoder.Results.SimID.7.apth5' #self.h5src
        fn = self.h5src
        hf = h5py.File( fn, 'r')
        #read ion type labels from HDF5 file using h5py into a numpy array and from this
        #use pandas to compile a pivot table for the composition
        pdfr = pd.DataFrame( columns=['IontypeID'], data = hf[PARAPROBE_RANGER_RES_TYPID][:,0] )
        pdfr['Cnts'] = 1
        cnts = pd.pivot_table( pdfr, values='Cnts', columns='IontypeID', aggfunc = "count" )
        del pdfr
        #replace the uint8 ion type ID keys which paraprobe uses internally to sth more human readable
        #eg the element names most likely associated with a certain element type
        #this is ranging information stored also in the h5src source file
        #paraprobe uses a particular format to encode molecular ions their elements and the charge
        ityp_hashvals = hf[PARAPROBE_RANGER_META_TYPID_DICT_WHAT][:,:]
        if np.shape(ityp_hashvals)[0] != cnts.shape[1]:
            raise ValueError('Data in ' + fn + ' are inconsistent: number of iontypes referred to in ' + PARAPROBE_RANGER_RES_TYPID + ' doesnt match number of rows in ' + PARAPROBE_RANGER_META_TYPID_DICT_WHAT)
        else:
            newnames = {}
            for i in np.arange(0,np.shape(ityp_hashvals)[0]):
                newnames[i] = decode_molecularion( ityp_hashvals[i,:] )
            print(newnames)
            cnts = cnts.rename( columns = newnames )
        #get total ion count
        #cnts['Total'] = cnts.sum(axis=1)
        self.cnts = cnts

    def get_composition(self):
        #add total as a column
        #and compute naive composition
        #cnts.append( cnts.divide(cnts['Total']) )
        imgfn = (self.h5src + '.NaiveComposition').replace('/','.')
        imgttl = (self.h5src).replace('/','.').lstrip('.')
        #render the figure
        self.results['CompositionFigure'] = plot_naive_composition( imgfn, self.cnts, imgttl )
        self.results['CompositionTable'] = self.cnts.to_dict()
        
    def report_mqhistogram(self, mqmin, mqicr, mqmax):
        #fn = '../HuanZhaoScrMatCaseStudy/PARAPROBE.Transcoder.Results.SimID.26120.h5'
        if mqmin < 0.0 or mqicr < 0.001 or mqmax <= mqmin:
            raise ValueError('We need to plot a finite mass-to-charge range and at least a binning width of 0.001 Dalton !')

        #for dsrg in self.h5src:
        #if self.ds_rg_n == 1:
        #fn = self.h5src[0][0]
        #fn = 'PARAPROBE.Transcoder.Results.SimID.26120.h5'
        #fn = dsrg[0]
        fn = self.h5src
        #mqmin = 0.0
        #mqicr = 0.001
        #mqmax = 140.0
        hf = h5py.File( fn, 'r')
        mq = hf[PARAPROBE_TRANS_RES_MQ][:,0]
        #build histogram by
        #first, filtering mass-to-charge values on [mqmin, mqmax]
        #second, binning and count summarize bin counts
        binIDs = np.uint64( (mq[np.logical_and(mq >= mqmin, mq <= mqmax)] + 0.5*mqicr ) / mqicr )#- mqmin
        hst1d = np.unique( binIDs, return_counts=True)
        #replace 0.0 with low number to be able to compute a log10
        x = np.float64(hst1d[0][:] * mqicr) #back to Da
        y = np.float64(hst1d[1][:])
        
        imgfn = fn + '.MassToCharge'
        imgttl = fn
        
        self.results['MassToChargeFigure'] = plot_naive_mass2charge_histogram( imgfn, x, y, imgttl )
        
    def report(self):
        return self.results

# =============================================================================
#     def decode_molecularion(self, hashval):
#         """
#         low-level function specific for paraprobe-ranger which extracts a human-readable
#         molecular ion string description from an np.uint8 type and shaped (8,) numpy array using periodictable
#         """
#         #first six column values give Z/N pair of first, second, and third isotope component of the molecular ion
#         #seventh column gives sign and eigth the charge of the molecular ion
#         #first one is either 0 indicating UNKNOWN_IONTYPE, i.e. unkwown, or single isotope ion
#         if hashval[0] == 0:
#             print(str('unknown'))
#             return 'unknown'
#         else:
#             #hashval = ityp_hashvals[1,:]
#             #tmp = '['  #use later for r'''$\textnormal{[}{\textnormal{Al}}_2\textnormal{O}]^{3+}$''', unlikely example though...
#             tmp = ''
#             if hashval[1] == 0:
#                 tmp += str(pse.elements[hashval[0]])
#             else:
#                 tmp += '^{'+ str(hashval[1]) + '}' + str(pse.elements[hashval[0]])
#                 #can use latex notation, no fuss any more with formatting :), see the beauty here
#             #tmp += ']'
#             ###MK::add the charges
#             print(str(tmp))
#             return tmp
# 
# =============================================================================

# =============================================================================
#example usage, tell me how many ions of a certain type
#task = autoreporter_ranger( 'PARAPROBE.Transcoder.Results.SimID.7.apth5' )
#show me the composition
#task.cnts
#task.report_natoms()
# =============================================================================
